function [tstr] = Image_Feature_learning_StackPSD_server(foldernames,imageformat,winsize,colorimage,npatches,outputfile,downsample,numlayers,nbasis_vec,lambda_vec,nepoch_vec,optmethod,whitenning,initialization)
%Function to learn the features.
%Input: foldernames - folders containing images used for feature learning,
%imageformat - Vector containing formats of the images to be read
%winsize - window size (square)
%colorimage - true if colorimage
%npatches - number of patches per image
%nbasis - number of basis functions to be learnt
%outputfile - name of file into which 'W' has to be written
disp('Feature Learning');
if ~exist('foldernames','var')
    disp(['error! folder directory not designated'])
end

if ~exist('imageformat','var')
    imageformat{1} = 'bmp';
end

if ~exist('winsize','var')
    winsize = 20;
end

if ~exist('colorimage','var')
    colorimage = false;
end

if ~exist('npatches','var')
    npatches = 50;
end

if ~exist('outputfile','var')
    outputfile = 'Stacked_PSD_numlayers_';
end

if ~strcmp(imageformat,'ids')
    idsformat = false;
else
    idsformat = true;
end

Y = [];
for i = 1:length(foldernames)
   for j = 1:length(imageformat)
     Y1 = Generate_Input_feature_learning(foldernames{i}, winsize,colorimage,idsformat,npatches,imageformat{j},downsample);
     Y = [Y Y1];
   end
end

if numlayers ~= length(nbasis_vec)
    disp(['error! number of layers does not equal to the length of nbasis_vec'])
    return
end

if whitenning == 1
    [Y, mean_patch, V] = preprocess(Y,2,1e-4);
else
    mean_patch = 0;
    V = 0;
end

W_layer = cell(numlayers,1);
D_layer = cell(numlayers,1);
G_layer = cell(numlayers,1);
X_layer = cell(numlayers,1);
Input_layer = cell(numlayers,1);
Input_layer{1,1} = Y;

scmethod = 0;
% optmethod = 0;
% initialization = 0;
tauw = 0.01;

for layer_idx = 1:numlayers
    disp(['Learning layer ' num2str(layer_idx)]);
    nbasis = nbasis_vec(layer_idx);
    lambda = lambda_vec(layer_idx);
    nepoch = nepoch_vec(layer_idx);
    Input = Input_layer{layer_idx,1}; %<-------- check last output X_last layer
    [W_layer{layer_idx,1} D_layer{layer_idx,1} G_layer{layer_idx,1} X_layer{layer_idx,1}] = LayerTrain_v2_server(Input,nbasis,lambda,nepoch,scmethod,optmethod,tauw,initialization); %<-------- check layer index
    Input_layer{layer_idx+1,1} = X_layer{layer_idx,1}; %<-------- check layer index    
end
tstr = datestr(now,'_yyyymmdd_HHMMSS');
save([outputfile num2str(numlayers) tstr '.mat'],'W_layer','D_layer','G_layer','X_layer','Input_layer','mean_patch','V');
return;

